#!/usr/bin/perl

use sigtrap;
use Socket;
use Image::Magick; 
use Net::FTP;
use POSIX qw( setsid );
use Date::Format;
use AppConfig;

#########################################################################################
#header start at: x=130  y=40
#footer start at: x=130  y=115
# Servers ip start at x=140 y=60
#Bugs? New features? Mail me at bruj0@moviegalaxy.com.ar
#
#(c)Copyright 2002 Rodrigo A. Diaz Leven a.k.a Bruj0 
#This code is released under the GPL license
#

my $config = AppConfig->new({
CREATE => 1,
ERROR => \&debug,
GLOBAL => {
    DEFAULT  => "<undef>",
    ARGCOUNT => ARGCOUNT_ONE,
}
},
                           
'font' => {
    DEFAULT =>  '-monotype-arial-medium-r-normal-*-*-150-*-*-p-*-iso8859-15',
    ARGCOUNT => ARGCOUNT_ONE,
},
'hffont' => {
    DEFAULT => '-monotype-arial-bold-r-normal-*-*-150-*-*-p-*-iso8859-15',
    ARGCOUNT => ARGCOUNT_ONE,
},
'path' => {
    DEFAULT => "/lab/mlservspy/",
    ARGCOUNT => ARGCOUNT_ONE,    
},
'image_file' => {
    DEFAULT => "mlservspy.jpg",
    ARGCOUNT => ARGCOUNT_ONE,    
},
'upload_file' => {
    DEFAULT => "sig.jpg",
    ARGCOUNT => ARGCOUNT_ONE,    
},
'color' => {
    DEFAULT => "red",
    ARGCOUNT => ARGCOUNT_ONE,    
},
'header' => {
    DEFAULT => "",
    ARGCOUNT => ARGCOUNT_ONE,
},
'offline_notice' => {
    DEFAULT => "The user is currently OFFLINE",
    ARGCOUNT => ARGCOUNT_ONE,
},
'footer' => {
    DEFAULT => "Last updated:",
    ARGCOUNT => ARGCOUNT_ONE,        
},
'mldonkey_host' => {
    DEFAULT => "127.0.0.1",
    ARGCOUNT => ARGCOUNT_ONE,
},
'mldonkey_port' => {
    DEFAULT => "4000",
    ARGCOUNT => ARGCOUNT_ONE,
},
'mldonkey_password' => {
    DEFAULT => 0,
    ARGCOUNT => ARGCOUNT_ONE,
},

'timetosleep' => {
    DEFAULT => '120',
    ARGCOUNT => ARGCOUNT_ONE,        
},
'debug' => {
    DEFAULT => 1,
    ARGCOUNT => ARGCOUNT_NONE,
},
'passive' => {
    DEFAULT => 1,
    ARGCOUNT => ARGCOUNT_NONE,
},
'ftp_hostname' => {
    DEFAULT => '',
    ARGCOUNT => ARGCOUNT_ONE,        
},

'ftp_username' => {
    DEFAULT => '',
    ARGCOUNT => ARGCOUNT_ONE,        
},
'ftp_directory' => {
    DEFAULT => '',
    ARGCOUNT => ARGCOUNT_ONE,        
},
'ftp_password' => {
    DEFAULT => '',
    ARGCOUNT => ARGCOUNT_ONE,        
},
'logfile' => {
    DEFAULT => 'mlservspy.log',
    ARGCOUNT => ARGCOUNT_ONE,        
});

$config->file('mlservspy.ini');
$path         =$config->path(); #full path to the directory where the file are written with a / at the end
$font         =$path.$config->font(); ## normal font
$hffont       ='@'.$path.$config->hffont(); ## font for header and footer
$image_file   =$path.$config->image_file(); ## background file
$upload_file  =$path.$config->upload_file(); ## name of the file to upload to a ftp ie: name of your signature
$color        =$config->color(); #color of the servers
$header       =$config->header();
$footer       =$config->footer(); ## the current time will be appended
$offline      =$config->offline_notice();
$host         =$config->mldonkey_host(); ## host where mldonkey is located
$mldonkey_port=$config->mldonkey_port();
$mldonkey_pass=$config->mldonkey_password();
$timetosleep  =$config->timetosleep(); ## time for the refresh in SECONDS
$debug        =$config->debug(); # set to 0 for no output
### FTP VARS
$passive      =$config->passive(); #set to 1 if you have a firewall
$hostname     =$config->ftp_hostname();
$username     =$config->ftp_username();
$password     =$config->ftp_password();
$upload_dir   =$config->ftp_directory();
$logfile      =$path.$config->logfile();
#########################################################################################
### Forking aka becoming a daemon thank for this code to: http://www.perlmonks.org/index.pl?node_id=121604
$|= 1;
my @fh_unused = (\*STDIN, \*STDOUT);
open \*STDERR, ">> $logfile";
select((select(\*STDERR), $| = 1)[0]);
{
# Daemon Rule 1) Fork and exit the parent.
    my $ppid = $$;
    my $pid = fork and exit 0;
    ! defined $pid and die "No Fork: ", $!;
    while (kill 0, $ppid) {
        select undef, undef, undef, .001;
    };
}

# Daemon Rule 2) become session leader, pg leader, no term
my $session_id = POSIX::setsid();

# Daemon Rule 3) cd to /
chdir '/' or die "Could not cd to rootfs", $!;

# Daemon Rule 4) set file creation mask to 0
my $oldmask = umask 00;

# Daemon Rule 5) Close unneeded file handles
close $_ or die $! for @fh_unused;
############################################################################################
sub debug
{
    my $format = shift;
    my ($str) =@_;
    if ($debug==1) { printf STDERR "$format",$str; }
}
###################################3
$iaddr = inet_aton($host)			or die "Unknown host: $host\n";
$paddr = sockaddr_in($mldonkey_port, $iaddr)		or die "getprotobyname: $!\n";
$proto = getprotobyname('tcp')			or die "getprotobyname: $!\n";
while(1)
{
    $date=time2str("%a %b %d %T GMT %z\n",time);

    open(IMAGE, $image_file) or die("Coundnt open $image_file");
    $image = Image::Magick->new;
    $image->Read(file=>\*IMAGE);

    socket(SOCKET, PF_INET, SOCK_STREAM, $proto)	or die "socket: $!\n";
    if(connect(SOCKET, $paddr))
    {

        while($recvbuf !~ /^Use/ )
        {
            $recvbuf = <SOCKET>;
        }

        if($mldonkey_pass)
        {
            debug("sent password ******\n");
            send(SOCKET, "auth $mldonkey_pass\n",0);
            $recvbuf = <SOCKET>;

        }

        send(SOCKET, "vm\n",0);

        $recvbuf = <SOCKET>;
        $recvbuf = <SOCKET>;

        $recvbuf =~ /(Connected to\s+(\d+))/;
        $num = $2;
        debug("\nNumber of servers=$num\n");

        if($num>6) {$num=6};
        for($i=1;$i<=$num;$i++)
        {
            $recvbuf = <SOCKET>;
            debug("Server $i=".$recvbuf."\n");
            $recvbuf =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)\:(\d+)/;
            $ip="$1.$2.$3.$4:$5";

            if($i>3) {
                $x=340;
                $y=($i-3)*20+40;
            }
            else {
                $x=140;
                $y=$i*20+40;
            }

            $image->Annotate(font=>$font, pointsize=>15,fill=>$color, text=>$ip, x=>$x,y=>$y);
        }
    }
    else
    {
    debug("STATUS IS OFFLINE\n");
    $image->Annotate(font=>$hffont,  pointsize=>25,fill=>$color, text=>$offline,x=>130,y=>80);
    }
    $image->Annotate(font=>$hffont,  pointsize=>15,fill=>black, text=>$header, x=>130,y=>40);
    $image->Annotate(font=>$hffont,  pointsize=>15,fill=>black, text=>$footer.$date, x=>130,y=>115);
    $image->Write($upload_file);
    close(IMAGE);
    close SOCKET;
    if ( $ftp=Net::FTP -> new($hostname,Debug => $debug,Passive => $passive) )
    {
        $ftp->login($username,$password);
        $ftp->binary;
        $ftp->cwd($upload_dir);
        $ftp->put($upload_file);
        $ftp->quit;
        debug("uploaded at $date\n");
    }
    else { debug("Connect failed to $hostname\n"); }
    sleep($timetosleep);
}


